""" Settings for core_explore_oaipmh_app

Settings with the following syntax can be overwritten at the project level:
SETTING_NAME = getattr(settings, "SETTING_NAME", "Default Value")
"""
from django.conf import settings

if not settings.configured:
    settings.configure()

INSTALLED_APPS = getattr(settings, "INSTALLED_APPS", [])



"""Explore OAI-PMH user views
"""
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.utils.rendering import render
from core_main_app.utils.view_builders import data as data_view_builder
from core_oaipmh_harvester_app.components.oai_record import api as oai_record_api


def data_detail(request):
    """Display data's detail for OAI-PMH

    Args:
        request:

    Returns:

    """
    try:
        record = oai_record_api.get_by_id(request.GET["id"], request.user)
        template = record.harvester_metadata_format.template

        data_object = {
            "record_id": record.id,
            "title": record.identifier,
            "xml_content": record.xml_content,
            "template": {
                "id": template.id if template is not None else "",
                "display_name": record.harvester_metadata_format.get_display_name(),
                "hash": record.harvester_metadata_format.hash,
            },
        }

        page_context = data_view_builder.build_page(data_object)

        return data_view_builder.render_page(
            request, render, "core_main_app/user/data/detail.html", page_context
        )
    except AccessControlError:
        error_message = "Access Forbidden"
        status_code = 403
    except Exception as e:
        error_message = "An error occurred: {0}".format(str(e))
        status_code = 400
    return render(
        request,
        "core_main_app/common/commons/error.html",
        context={"error": error_message, "status_code": status_code},
    )



"""Url router for the REST API
"""

from django.urls import re_path

from core_explore_oaipmh_app.rest.query import views as query_views
from core_explore_oaipmh_app.rest.result import views as result_views

urlpatterns = [
    re_path(
        r"^execute-query",
        query_views.ExecuteQueryView.as_view(),
        name="core_explore_oaipmh_rest_execute_query",
    ),
    re_path(
        r"^result",
        result_views.get_result_from_data_id,
        name="core_explore_oaipmh_app_rest_get_result_from_data_id",
    ),
]



""" Fixtures files for Data
"""
from core_explore_common_app.components.query.models import Query
from core_main_app.components.data.models import Data
from core_main_app.components.template.models import Template
from core_main_app.components.workspace.models import Workspace
from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface


class AccessControlDataFixture(FixtureInterface):
    """Access Control Data fixture"""

    template = None
    workspace_user1 = None
    public_workspace = None
    data_collection = None
    data_workspace_1 = None
    data_no_workspace = None
    data_public_workspace = None
    query_user1 = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_workspaces()
        self.generate_data_collection()
        self.generate_queries()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """
        content = {"root": {"element": "value2"}}

        self.data_no_workspace = Data(
            template=self.template, title="Data 1", user_id="1"
        )
        self.data_no_workspace.save()
        self.data_workspace_1 = Data(
            template=self.template,
            title="Data 3",
            user_id="1",
            workspace=self.workspace_user1,
            dict_content=content,
        )
        self.data_workspace_1.save()
        self.data_public_workspace = Data(
            template=self.template,
            title="DataDoubleTitle",
            user_id="2",
            workspace=self.public_workspace,
        )
        self.data_public_workspace.save()

        self.data_collection = [
            self.data_no_workspace,
            self.data_workspace_1,
            self.data_public_workspace,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        template.content = xsd
        template.hash = ""
        template.filename = "filename"
        self.template = template
        self.template.save()

    def generate_workspaces(self):
        """Generate the workspaces.

        Returns:

        """
        self.workspace_user1 = Workspace(
            title="Workspace 1", owner="1", read_perm_id="1", write_perm_id="1"
        )
        self.workspace_user1.save()
        self.public_workspace = Workspace(
            title="public",
            owner="1",
            read_perm_id="3",
            write_perm_id="3",
            is_public=True,
        )
        self.public_workspace.save()

    def generate_queries(self):
        """Generate queries

        Returns:

        """
        self.query_user1 = Query(user_id="1")
        self.query_user1.save()



""" Ajax User core explore OAI-PMH
"""
import json

from django.http import HttpResponseForbidden
from django.http.response import HttpResponseBadRequest, HttpResponse
from django.shortcuts import render
from django.urls import reverse
from django.utils.html import escape

import core_explore_common_app.components.query.api as api_query
from core_explore_common_app.components.abstract_query.models import (
    Authentication,
    DataSource,
)
from core_explore_oaipmh_app import settings
from core_explore_oaipmh_app.components.query import api as api_oaipmh_query
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.settings import DATA_SORTING_FIELDS
from core_oaipmh_harvester_app.components.oai_registry import api as oai_registry_api


def get_data_source_list_oaipmh(request):
    """Ajax method to fill the list of data sources.

    Args:
        request:

    Returns:

    """
    try:
        id_query = request.GET.get("id_query", None)

        if id_query is not None:
            # Get query from id
            query = api_query.get_by_id(id_query, request.user)
            instance_list = oai_registry_api.get_all_activated_registry(
                order_by_field="name"
            )
            item_list = []
            for instance_item in instance_list:
                checked = False
                # compare instance with existing data source in query
                # in order to know if they have to be checked
                for data_source_item in query.data_sources:
                    if "instance_id" in data_source_item[
                        "query_options"
                    ] and data_source_item["query_options"]["instance_id"] == str(
                        instance_item.id
                    ):
                        checked = True

                # update the result item list for the context
                item_list.extend(
                    [
                        {
                            "instance_id": instance_item.id,
                            "instance_name": instance_item.name,
                            "is_checked": checked,
                        }
                    ]
                )

            # Here, data sources are instances
            context_params = dict()
            context_params["instances"] = item_list

            # return context
            context = {}
            context.update(request)
            context.update(context_params)
            return render(
                request,
                "core_explore_oaipmh_app/user/data_sources/list-content.html",
                context,
            )
        else:
            return HttpResponseBadRequest(
                "Error during loading data sources from oaipmh search."
            )
    except AccessControlError:
        return HttpResponseForbidden()
    except Exception as e:
        return HttpResponseBadRequest(
            "Error during loading data sources from oaipmh search: %s" % escape(str(e))
        )


def update_data_source_list_oaipmh(request):
    """Ajax method to update query data sources in data base.

    Args:
        request:

    Returns:

    """
    try:
        id_query = request.GET.get("id_query", None)
        id_instance = request.GET.get("id_instance", None)
        added = request.GET.get("to_be_added", None)
        to_be_added = json.loads(added) if added is not None else False

        # Get query from id
        if id_query is not None:
            query = api_query.get_by_id(id_query, request.user)
            url_instance = request.build_absolute_uri(
                reverse("core_explore_oaipmh_rest_execute_query")
            )
            instance = oai_registry_api.get_by_id(id_instance)
            if to_be_added:
                # Instance have to be added in the query as a data source
                authentication = Authentication(auth_type="session")
                data_source = DataSource(
                    name=instance.name,
                    url_query=url_instance,
                    authentication=authentication,
                    order_by_field=",".join(DATA_SORTING_FIELDS),
                    query_options={"instance_id": str(instance.id)},
                )

                if "core_linked_records_app" in settings.INSTALLED_APPS:
                    data_source.capabilities = {
                        "url_pid": request.build_absolute_uri(
                            reverse("core_linked_records_app_query_oaipmh")
                        )
                    }

                api_oaipmh_query.add_oaipmh_data_source(
                    query, data_source, request.user
                )
            else:
                api_oaipmh_query.remove_oaipmh_data_source(
                    query, id_instance, request.user
                )

            return HttpResponse()
        else:
            return HttpResponseBadRequest("Error during data source selection.")
    except AccessControlError:
        return HttpResponseForbidden()
    except Exception as e:
        return HttpResponseBadRequest(
            "Error during data source selection: %s" % escape(str(e))
        )



""" Url router for the explore OAI-PMH search application
"""
from django.conf.urls import include
from django.urls import re_path

import core_explore_oaipmh_app.views.user.ajax as user_ajax
import core_explore_oaipmh_app.views.user.views as user_views

urlpatterns = [
    re_path(
        r"^get_data_sources",
        user_ajax.get_data_source_list_oaipmh,
        name="core_explore_oaipmh_app_get_data_sources",
    ),
    re_path(
        r"^update_data_sources",
        user_ajax.update_data_source_list_oaipmh,
        name="core_explore_oaipmh_app_update_data_sources",
    ),
    re_path(
        r"^data", user_views.data_detail, name="core_explore_oaipmh_app_data_detail"
    ),
    re_path(r"^rest/", include("core_explore_oaipmh_app.rest.urls")),
]



""" REST views for the data API
"""

from rest_framework import status
from rest_framework.decorators import api_view, schema
from rest_framework.response import Response

from core_explore_common_app.components.result.models import Result
from core_explore_common_app.rest.result.serializers import ResultSerializer
from core_main_app.commons.exceptions import DoesNotExist
from core_oaipmh_harvester_app.components.oai_record import api as oai_record_api


@api_view(["GET"])
@schema(None)
def get_result_from_data_id(request):
    """Access data, Returns Result, Expects a data ID

    Args:
        request:

    Returns:

    """
    try:
        # get parameters
        data_id = request.GET.get("id", None)
        # if no data id given
        if data_id is None:
            content = {"message": "Data id is missing"}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)

        # reverse url for accessing data
        record = oai_record_api.get_by_id(data_id, request.user)
        # No title for OaiRecord. Use of the id.
        result = Result(title=str(data_id), xml_content=record.xml_content)
        # Serialize results
        return_value = ResultSerializer(result)
        # Returns the response
        return Response(return_value.data, status=status.HTTP_200_OK)
    except DoesNotExist as e:
        # The record doesn't exist with this id
        content = {"message": "No Record found with the given id."}
        return Response(content, status=status.HTTP_404_NOT_FOUND)
    except Exception as e:
        # if something went wrong, return an internal server error
        content = {"message": str(e)}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



""" Permission tests on views
"""
from django.test import RequestFactory

from core_explore_oaipmh_app.views.user.ajax import (
    get_data_source_list_oaipmh,
    update_data_source_list_oaipmh,
)
from core_explore_oaipmh_app.views.user.views import data_detail
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from tests.fixtures.fixtures import AccessControlDataFixture


class TestGetDataSourceListOaipmh(MongoIntegrationBaseTestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = create_mock_user(user_id=None, is_anonymous=True)
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_access_data_sources_of_a_user_query(self):
        request = self.factory.get("core_explore_oaipmh_app_get_data_sources")
        request.user = self.anonymous
        request.GET = {"id_query": str(self.fixture.query_user1.id)}
        response = get_data_source_list_oaipmh(request)
        self.assertEqual(response.status_code, 403)


class TestUpdateDataSourceListOaipmh(MongoIntegrationBaseTestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = create_mock_user(user_id=None, is_anonymous=True)
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_access_data_sources_of_a_user_query(self):
        request = self.factory.get("core_explore_oaipmh_app_update_data_sources")
        request.user = self.anonymous
        request.GET = {"id_query": str(self.fixture.query_user1.id)}
        response = update_data_source_list_oaipmh(request)
        self.assertEqual(response.status_code, 403)


class TestViewData(MongoIntegrationBaseTestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = create_mock_user(user_id=None, is_anonymous=True)
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_access_a_data_that_is_not_in_a_workspace(self):
        request = self.factory.get("core_explore_oaipmh_app_data_detail")
        request.user = self.anonymous
        request.GET = {
            "id": str(self.fixture.data_no_workspace.id),
        }
        response = data_detail(request)
        self.assertTrue(
            self.fixture.data_no_workspace.title not in response.content.decode()
        )
        self.assertTrue("Error 403" in response.content.decode())

    def test_an_anonymous_user_can_not_access_a_data_that_is_in_a_private_workspace(
        self,
    ):
        request = self.factory.get("core_explore_oaipmh_app_data_detail")
        request.user = self.anonymous
        request.GET = {
            "id": str(self.fixture.data_workspace_1.id),
        }
        response = data_detail(request)
        self.assertTrue(
            self.fixture.data_workspace_1.title not in response.content.decode()
        )
        self.assertTrue("Error 403" in response.content.decode())

    def test_an_anonymous_user_can_not_access_a_data_that_is_in_a_public_workspace_and_access_setting_is_false(
        self,
    ):
        request = self.factory.get("core_explore_oaipmh_app_data_detail")
        request.user = self.anonymous
        request.GET = {
            "id": str(self.fixture.data_public_workspace.id),
        }
        response = data_detail(request)
        self.assertTrue(
            self.fixture.data_public_workspace.title not in response.content.decode()
        )
        self.assertTrue("Error 403" in response.content.decode())
